"use strict";
// Copyright 2016-2018, Pulumi Corporation.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
Object.defineProperty(exports, "__esModule", { value: true });
const azure = require("@pulumi/azure");
const pulumi = require("@pulumi/pulumi");
const fs = require("fs");
const filepath = require("path");
const util_1 = require("./util");
/**
 * Takes in a callback and a set of bindings, and produces the right AssetMap layout that Azure
 * FunctionApps expect.
 */
function serializeCallback(name, handler, bindingsOutput) {
    const includedPackages = new Set();
    const excludedPackages = new Set();
    const pathSetOutput = pulumi.output(allFoldersForPackages(includedPackages, excludedPackages));
    const serializedHandlerOutput = pulumi.output(pulumi.runtime.serializeFunction(handler));
    return pulumi.all([bindingsOutput, serializedHandlerOutput, pathSetOutput]).apply(([bindings, serializedHandler, pathSet]) => {
        const map = {};
        map["host.json"] = new pulumi.asset.StringAsset(JSON.stringify({
            "tracing": {
                "consoleLevel": "verbose",
            },
        }));
        map[`${name}/function.json`] = new pulumi.asset.StringAsset(JSON.stringify({
            "disabled": false,
            "bindings": bindings,
        }));
        map[`${name}/index.js`] = new pulumi.asset.StringAsset(`module.exports = require("./handler").handler`),
            map[`${name}/handler.js`] = new pulumi.asset.StringAsset(serializedHandler.text);
        // TODO: unify this code with aws-serverless instead of straight copying.
        // For each of the required paths, add the corresponding FileArchive or FileAsset to the AssetMap.
        for (const path of pathSet.values()) {
            // The Asset model does not support a consistent way to embed a file-or-directory into an `AssetArchive`, so
            // we stat the path to figure out which it is and use the appropriate Asset constructor.
            const stats = fs.lstatSync(path);
            if (stats.isDirectory()) {
                map[name + "/" + path] = new pulumi.asset.FileArchive(path);
            }
            else {
                map[name + "/" + path] = new pulumi.asset.FileAsset(path);
            }
        }
        return map;
    });
}
const readPackageTree = require("read-package-tree");
// allFolders computes the set of package folders that are transitively required by the root
// 'dependencies' node in the client's project.json file.
function allFoldersForPackages(includedPackages, excludedPackages) {
    return new Promise((resolve, reject) => {
        readPackageTree(".", undefined, (err, root) => {
            if (err) {
                return reject(err);
            }
            // This is the core starting point of the algorithm.  We use readPackageTree to get the
            // package.json information for this project, and then we start by walking the
            // .dependencies node in that package.  Importantly, we do not look at things like
            // .devDependencies or or .peerDependencies.  These are not what are considered part of
            // the final runtime configuration of the app and should not be uploaded.
            const referencedPackages = new Set(includedPackages);
            if (root.package && root.package.dependencies) {
                for (const depName of Object.keys(root.package.dependencies)) {
                    referencedPackages.add(depName);
                }
            }
            const packagePaths = new Set();
            for (const pkg of referencedPackages) {
                addPackageAndDependenciesToSet(root, pkg, packagePaths, excludedPackages);
            }
            resolve(packagePaths);
        });
    });
}
// addPackageAndDependenciesToSet adds all required dependencies for the requested pkg name from the given root package
// into the set.  It will recurse into all dependencies of the package.
function addPackageAndDependenciesToSet(root, pkg, packagePaths, excludedPackages) {
    // Don't process this packages if it was in the set the user wants to exclude.
    // Also, exclude it if it's an @pulumi package.  These packages are intended for deployment
    // time only and will only bloat up the serialized lambda package.
    if (excludedPackages.has(pkg) ||
        pkg.startsWith("@pulumi")) {
        return;
    }
    const child = findDependency(root, pkg);
    if (!child) {
        console.warn(`Could not include required dependency '${pkg}' in '${filepath.resolve(root.path)}'.`);
        return;
    }
    packagePaths.add(child.path);
    if (child.package.dependencies) {
        for (const dep of Object.keys(child.package.dependencies)) {
            addPackageAndDependenciesToSet(child, dep, packagePaths, excludedPackages);
        }
    }
}
// findDependency searches the package tree starting at a root node (possibly a child) for a match for the given name.
// It is assumed that the tree was correctly construted such that dependencies are resolved to compatible versions in
// the closest available match starting at the provided root and walking up to the head of the tree.
function findDependency(root, name) {
    for (; root; root = root.parent) {
        for (const child of root.children) {
            let childName = child.name;
            // Note: `read-package-tree` returns incorrect `.name` properties for packages in an orgnaization - like
            // `@types/express` or `@protobufjs/path`.  Compute the correct name from the `path` property instead. Match
            // any name that ends with something that looks like `@foo/bar`, such as `node_modules/@foo/bar` or
            // `node_modules/baz/node_modules/@foo/bar.
            const childFolderName = filepath.basename(child.path);
            const parentFolderName = filepath.basename(filepath.dirname(child.path));
            if (parentFolderName[0] === "@") {
                childName = filepath.join(parentFolderName, childFolderName);
            }
            if (childName === name) {
                return child;
            }
        }
    }
    return undefined;
}
/**
 * Base type for all subscription types.
 */
class EventSubscription extends pulumi.ComponentResource {
    constructor(type, name, callback, bindings, args, options) {
        super(type, name, {}, options);
        const parentArgs = { parent: this };
        const appSettings = args.appSettings || pulumi.output({});
        if (!args.resourceGroup) {
            throw new pulumi.RunError("[resourceGroup] must be provided in [args].");
        }
        this.resourceGroup = pulumi.output(args.resourceGroup);
        const resourceGroupName = this.resourceGroup.apply(g => g.name);
        const location = this.resourceGroup.apply(g => g.location);
        const resourceGroupArgs = {
            resourceGroupName,
            location,
        };
        this.storageAccount = args.storageAccount || new azure.storage.Account(`${name}`, Object.assign({}, resourceGroupArgs, { accountKind: "StorageV2", accountTier: "Standard", accountReplicationType: "LRS" }), parentArgs);
        this.storageContainer = args.storageContainer || new azure.storage.Container(`${name}`, {
            resourceGroupName: resourceGroupName,
            storageAccountName: this.storageAccount.name,
            containerAccessType: "private",
        }, parentArgs);
        this.appServicePlan = args.appServicePlan || new azure.appservice.Plan(`${name}`, Object.assign({}, resourceGroupArgs, { kind: "FunctionApp", sku: {
                tier: "Dynamic",
                size: "Y1",
            } }), parentArgs);
        const assetMap = serializeCallback(name, callback, bindings);
        // const appSettings = assetAndAppSettings.apply(a => a.appSettings);
        const blob = new azure.storage.ZipBlob(name, {
            resourceGroupName: resourceGroupName,
            storageAccountName: this.storageAccount.name,
            storageContainerName: this.storageContainer.name,
            type: "block",
            content: assetMap.apply(m => new pulumi.asset.AssetArchive(m)),
        }, parentArgs);
        const codeBlobUrl = util_1.signedBlobReadUrl(blob, this.storageAccount, this.storageContainer);
        this.functionApp = new azure.appservice.FunctionApp(name, Object.assign({}, resourceGroupArgs, { appServicePlanId: this.appServicePlan.id, storageConnectionString: this.storageAccount.primaryConnectionString, appSettings: appSettings.apply(settings => (Object.assign({}, settings, { "WEBSITE_RUN_FROM_ZIP": codeBlobUrl }))) }), parentArgs);
    }
}
exports.EventSubscription = EventSubscription;
//# sourceMappingURL=subscription.js.map